// Substitution (expression distribution for variables) -*- c++ -*-

#ifdef __GNUC__
# pragma implementation
#endif // __GNUC__
#include "Substitution.h"
#include "Expression.h"
#include "VariableDefinition.h"

/** @file Substitution.C
 * Map from variables (quantifiers or function parameters) to expressions
 */

Substitution::Substitution () :
  myExprs ()
{
}

Substitution::~Substitution ()
{
  for (iterator i = myExprs.begin (); i != myExprs.end (); i++)
    i->second->destroy ();
}

class Expression*
Substitution::getExpr (const class VariableDefinition& var)
{
  const_iterator i = myExprs.find (&var);
  return i == myExprs.end () ? NULL : i->second->copy ();
}

void
Substitution::setExpr (const class VariableDefinition& var,
		       class Expression& expr)
{
  assert (expr.isAssignable (var.getType ()));
  assert (!getExpr (var));
  myExprs[&var] = &expr;
}
