// Value base class -*- c++ -*-

#ifdef __GNUC__
# pragma implementation
#endif // __GNUC__
#include "allValues.h"
#include "UnionType.h"

/** @file Value.C
 * Abstract base class for values
 */

bool
Value::operator< (const class Value& other) const
{
  assert (other.getKind () == getKind ());
  switch (getKind ()) {
  case vLeaf:
    return static_cast<const class LeafValue&>(*this) <
      static_cast<const class LeafValue&>(other);
  case vStruct:
    return static_cast<const class StructValue&>(*this) <
      static_cast<const class StructValue&>(other);
  case vUnion:
    return static_cast<const class UnionValue&>(*this) <
      static_cast<const class UnionValue&>(other);
  case vVector:
    return static_cast<const class VectorValue&>(*this) <
      static_cast<const class VectorValue&>(other);
  case vBuffer:
    return static_cast<const class BufferValue&>(*this) <
      static_cast<const class BufferValue&>(other);
  }
  assert (false);
  return false;
}

bool
Value::operator== (const class Value& other) const
{
  assert (other.getKind () == getKind ());
  switch (getKind ()) {
  case vLeaf:
    return static_cast<const class LeafValue&>(*this) ==
      static_cast<const class LeafValue&>(other);
  case vStruct:
    return static_cast<const class StructValue&>(*this) ==
      static_cast<const class StructValue&>(other);
  case vUnion:
    return static_cast<const class UnionValue&>(*this) ==
      static_cast<const class UnionValue&>(other);
  case vVector:
    return static_cast<const class VectorValue&>(*this) ==
      static_cast<const class VectorValue&>(other);
  case vBuffer:
    return static_cast<const class BufferValue&>(*this) ==
      static_cast<const class BufferValue&>(other);
  }
  assert (false);
  return false;
}

card_t
Value::operator- (const class Value& other) const
{
  assert (other.getKind () == getKind ());
  assert (!(*this < other));
  switch (getKind ()) {
  case vLeaf:
    return static_cast<const class LeafValue&>(*this) -
      static_cast<const class LeafValue&>(other);
  case vStruct:
    return static_cast<const class StructValue&>(*this) -
      static_cast<const class StructValue&>(other);
  case vUnion:
    return static_cast<const class UnionValue&>(*this) -
      static_cast<const class UnionValue&>(other);
  case vVector:
    return static_cast<const class VectorValue&>(*this) -
      static_cast<const class VectorValue&>(other);
  case vBuffer:
    return static_cast<const class BufferValue&>(*this) -
      static_cast<const class BufferValue&>(other);
  }
  assert (false);
  return 0;
}

class Value*
Value::cast (const class Type& type)
{
  if (type.getKind () == Type::tUnion)
    return static_cast<const class UnionType&>(type).cast (*this);

  if (getKind () != vLeaf ||
      (type.getKind () == myType->getKind () ||
       int_t (static_cast<class LeafValue&>(*this)) >= 0 ||
       myType->getKind () != Type::tInt ||
       type.getKind () == Type::tInt)) {
    setType (type);
    if (type.isConstrained (*this))
      return this;
  }

  delete this;
  return 0;
}
